/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * ModuleStructure.js
 * (c) Copyright IBM Corporation 2011, 2014.
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
	
dojo.provide("com.ibm.rdm.web.model.resources.modules.ModuleStructure");

dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.query.ViewServiceClient");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.model.resources.modules.Structure");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.model.resources.modules.SectionNumberingStrategy");
dojo.require("com.ibm.rdm.web.project.EditManager");
dojo.require("com.ibm.rdm.web.model.resources.requirements.Requirement");
dojo.require("com.ibm.rdm.web.server.client.IndexingValidator");


var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes,
	MessageHandler = com.ibm.rdm.web.common.MessageHandler,
	ViewServiceClient = com.ibm.rdm.web.query.ViewServiceClient,
	DomUtils = com.ibm.rdm.web.util.domutils,
	ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier,
	SectionNumberingStrategy = com.ibm.rdm.web.model.resources.modules.SectionNumberingStrategy,
	EditManager = com.ibm.rdm.web.project.EditManager,
	RequirementSerializer = com.ibm.rdm.web.model.resources.requirements.Requirement.Serializer,
	IndexingValidator = com.ibm.rdm.web.server.client.IndexingValidator,
	REQUIRED_ATTRIBUTES = [
		RRMTypes.ETAG,
//		RRMTypes.REVISION,
		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier,
		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Depth,
		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Section,
		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ParentBinding,
//		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.PrimaryText,
//		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat,
		RRMTypes.WELL_KNOWN_IS_HEADING
//		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.WrappedResourceMimeType,
//		RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.WrappedResourceURI
	],
	DEFAULT_ORDER = {
		def: RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BookOrder,
		order:	"a"
	};


dojo.declare("com.ibm.rdm.web.model.resources.modules.ModuleStructure", null, {
	
	// summary:
	//		A simple class for fetching the structure of a module using the view service.
	// description:
	//		Stores the structure as a list of objects in the specified sort order.
	//		Clients that are passed a reference to instances of this object can check
	//		isLoaded to see if the list of artifacts is avaialble, and can connect to
	//		onLoad if not.  
	//		Use getArtifacts() to get the sorted array of artifacts.
	//		{
	//		    uri:
	//		    depth:
	//		    title:
	//		    sectionNumber:
	//		    summaryText:
	//		    parent:
	//		    children:
	//		}

	compositeRootURI: null,
	
	checkLatest: null,
	
	checkLatestTimestamp: null, 
	
	displayMergeConflict: null, 
	
	baselineURI: null,
	
	currentArtifactCount: null,
	
	structureResource: null,

	_saveListeners: null,
	
	_totalArtifactsInModule: null,
	
	// Current module levels to display.
	// Used for re-apply the hierarchy level after refresh.
	_currentTreeLevel: -1,
	
	constructor: function (args) {
		dojo.mixin(this, args);
		this.sectionNumberer = SectionNumberingStrategy.getDefault();
		this._deepestLevel = 0;
		this._saveListeners = [];
		this._saveListeners.push(ChangeNotifier.addListener({
			eventTopics: [
				com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_SAVED
				],
			callbackFunction: dojo.hitch(this, this._updateOnSave),
			callbackScope: this,
			projectURI: this.projectURI,
			location: 0 //Need to ensure that in this situation, this is THE FIRST thing fired
		}));
	},
	
	setBaseline: function (baselineURI) {
		this.baselineURI = baselineURI;
	},
	
	onLoad: function (args) {
		// summary:
		//		Stub fired when this object is loaded.
		// description:
		//		A client passed a reference a module structure can do something like:
		//		
		//		if (structure.isLoaded) {
		//			...do something with structure.
		//		} else {
		//			this.connect(structure, "onLoad", function () {
		//				...do something with structure.
		//			});
		//		}	
	},
	
	onChange: function() {
		// summary:
		//		Stub fired when a change is made to the structure.
		//		A client can connect to this to respond to changes in the
		//		structure.
		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.MODULE_STRUCTURE_CHANGED,
			resourceUriScope : this.moduleURI,
			projectURI : this.projectURI
		});            
	},
	
	onCountChanged: function() {
		// summary:
		//		Stub fired when a the currently viewed number of artifacts
		//		has changed. A client can connect to this to respond to changes
		//		in currently viewed artifacts.
		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.MODULE_COUNT_CHANGED,
			resourceUriScope: this.moduleURI,
			projectURI: this.projectURI
		});
	},
	
	setViewShapeQueryParams: function (params) {
		this.viewShapeQueryParams = params;
	},
	
	getViewShapeQueryParams: function () {
		return this.viewShapeQueryParams;
	},
	
	hasViewFiltersApplied: function () {
		var viewParams = this.getViewShapeQueryParams();
		if (viewParams) {
			if ((viewParams.folderURIs && viewParams.folderURIs.length > 0) ||
			(viewParams.tagURIs && viewParams.tagURIs.length > 0) ||
			viewParams.artifactTypeConditionInfo ||
			(viewParams.attributeConditionInfo && viewParams.attributeConditionInfo.length > 0) ||
			(viewParams.attributeConditionInfos && viewParams.attributeConditionInfos.length > 0) ||
			(viewParams.linkedArtifactConditionInfos && viewParams.linkedArtifactConditionInfos.length > 0)) {
				return true;
			}
		}
		return false;
	},
	
	isSortApplied: function () {
		// summary:
		//		Return true if any other order other than book order
		//		is applied.
		var sortBy = this.getPrimarySortBy();
		if (sortBy) {
			// return true if not book order asc.
			return ((sortBy.def !== DEFAULT_ORDER.def) || (sortBy.order !== DEFAULT_ORDER.order));
		}
	},
	
	getPrimarySortBy: function () {
		return (this.viewShapeQueryParams && this.viewShapeQueryParams.orderBy && this.viewShapeQueryParams.orderBy[0])?
			this.viewShapeQueryParams.orderBy[0]:
			DEFAULT_ORDER;
	},
	
	getSortBy: function () {
		return (this.viewShapeQueryParams && this.viewShapeQueryParams.orderBy && this.viewShapeQueryParams.orderBy[0])?
			this.viewShapeQueryParams.orderBy:
			[DEFAULT_ORDER];
	},
	
	load: function (args) {
		var viewServiceRequestParams;
		
		this.isLoaded = false;
		var order = this.getSortBy();

		if (this.viewShapeQueryParams) {
			viewServiceRequestParams = dojo.mixin(this.viewShapeQueryParams, {
				requestedAttributeUris: REQUIRED_ATTRIBUTES,
				artifactUris: null,
				pageRequested: null,
				pageSize: null,
				baselineURI: this.baselineURI,
				summary: true,
				showReused: true
			});
		} else {
			viewServiceRequestParams = {
				projectURI: this.projectURI,
				moduleURI: this.moduleURI,
				baselineURI: this.baselineURI,
				summary: true,
				orderBy: order,
				requestedAttributeUris: REQUIRED_ATTRIBUTES,
				capQueryExecutionTime: true
			};
		}
		//We don't want lock info here ever
		viewServiceRequestParams.processLockInfo = null;
		
		// WI: 84633 - We must make sure we never ask for the
		// full XHTML text of the artifacts in the structure query, so 
		// always set the fetchLarge param to false.
		viewServiceRequestParams.fetchLarge = false;
		
		return ViewServiceClient.getViewContents(viewServiceRequestParams)
					.addBoth(dojo.hitch(this, function (response) {
						this._processViewServiceResponse(response, args);
					}));
	},
	
	
	_createArtifact: function (req) {
		// summary:
		//		Creates a simple object literal from the view service response
		//		The "req" argument may refer to a PartialRequirement (which is made
		//		when we get a response from the view service) or a Requirement which
		//		is when we are adding existing or new artifacts.

		var node = {
			id: req.getID() || req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier),
			sectionNumber: req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Section), 
			title: req.getTitle() || "",
			/* getPrimaryHTMLText() returns an non empty string, hence not falsy value */
			summary: req.getPrimaryText() ? req.getPrimaryHtmlText() : req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.PrimaryText) || "",
			uri: req.getResourceURI(),
			format: req.artifactFormat || req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat),
			depth: parseInt(req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Depth), 10),
			parentBinding: req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ParentBinding),
			isHeading: req.isHeading(),
			children: [],
			type: req.getAttributeValue(RRMTypes.WELL_KNOWN_OBJECT_TYPE),
			wrappedResourceMimeType: req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.WrappedResourceMimeType),
			wrappedResourceURI: req.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.WrappedResourceURI),
			eTag: req.getAttributeValue(RRMTypes.ETAG) || req.getETag(),
			isStub: ((req.getResourceURI() && req.getResourceURI().indexOf("STUB") )=== 0 ? true : false)
		};
		
		if (!isNaN(node.depth)) {
			this._deepestLevel = Math.max(this._deepestLevel, node.depth);
		}
		
		return node;
	},
	
	
	requestAndPlaceURIs: function (target, uris, where) {
		// summary:
		//		Request the given URIs and place in the structure
		//		relative to target.
		var viewServiceRequestParams = {
			projectURI: this.projectURI,
			moduleURI: this.moduleURI,
			summary: true,
			orderBy: [DEFAULT_ORDER],
			requestedAttributeUris: REQUIRED_ATTRIBUTES,
			artifactUris: uris,
			capQueryExecutionTime: true
		};
		
		var deferred = ViewServiceClient.getViewContents(viewServiceRequestParams);
		deferred.addCallback(this, function (response) {
			var parsedResult = ViewServiceClient.parseViewContentsResponse(response, null, {}, false),
				entries = [],
				i; // loop counters...
		
			// find out where our target is in the array now, before
			// we go adding children during treeification
			var targetEntry = this.artifactsMap[target];
			var idxOfTarget = this.indexOf(targetEntry);
						
			
			if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER) {
				if (target !== this.compositeRootURI) {
					idxOfTarget += this.countDescendants(targetEntry) + 1;
				} else {
					idxOfTarget = 0;
				}
			} else if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW) {
				idxOfTarget += 1;
			}
			
			for (i=0; i<parsedResult.resources.length; i++) {
				var entry = this._createArtifact(parsedResult.resources[i]);
				entries.push(entry);
				this.artifactsMap[entry.uri] = entry;
			}

			// build the sub structure into a tree, assigning the correct
			// parent and children references to the newly pasted artifacts
			this._treeify(entries);
			
			// need to make sure that the thing goes in the right place in it's parents children
			// the only thing we need to move is the root of te copied sub structure though...
			if ((where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER)
				|| (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE)) {
				var children = targetEntry.parent && targetEntry.parent.children;
				if (children) {
					// ensure target entry is expanded...
					targetEntry.collapsed = false;
					var idxOfTargetInChildren = dojo.indexOf(children, targetEntry);
					if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER) {
						idxOfTargetInChildren += 1;
					}
					// oops, entries[0] has already been added to the parent children[]
					// by treeify, we can probably fix this up, treeify is a bit too generic.
					children.pop(); 
					// popped it off, now put it in the right place
					children.splice(idxOfTargetInChildren, 0, entries[0]);
				}
			} else {
				// must be below....
				// make sure any entries (what was pasted) which are now direct children of target
				// are added to the start of the array of children (WI: 69968).
				var children = targetEntry ? targetEntry.children : [],
					toAddToStart = [],
					idx;
				dojo.forEach(entries, function(entry) {
					// ensure that any newly added artifacts that
					// are direct descendants of the target entry
					// are added to the start of the list.
					if (entry.parent === targetEntry) {
						idx = dojo.indexOf(children, entry);
						// the treeify function adds the entries to the end
						// of the list of children (perhaps it is too generic), 
						// for top level entries, we need them to be added
						// at the start.						
						children.splice(idx, 1);
						toAddToStart.push(entry);
					}
				});
				
				children = toAddToStart.concat(children);
				if (targetEntry) {
					targetEntry.children = children;
				}
			}
			
			
			// now, add this array into our list of artifacts, again use a trick
			// to make the array (entries) get added into the this.artifacts array
			// splice will by default make the argument an entry in the array, i.e.
			// we would end up with this.artifacts[idxOfTarget] = entries.
			Array.prototype.splice.apply(this.artifacts, [idxOfTarget, 0].concat(entries));
			// finally fire the change event after renumbering, this will prompt the grid
			// to refresh.
			this._numberAndFireOnChange({});
		
		});
		return deferred;
	},
	
	_treeify: function (entries) {
		var i, entry, parentBinding, parent;
		for (i=0; i<entries.length; i++) {
			entry = entries[i];
			parentBinding = entry.parentBinding;
			if (parentBinding) {
				parent = this.artifactsMap[parentBinding];
				if (parent) {
					entry.parent = parent;
					parent.children.push(entry);
				}
			}
		} 		
	},
	
	_processViewServiceResponse: function (response, args) {
		
		var parsedResult = ViewServiceClient.parseViewContentsResponse(response, null, {}, false),
			entries = [],
			i; // loop counters...

		if (!this.hasViewFiltersApplied()) {
			this._totalArtifactsInModule = parsedResult.resources.length;
		}
		
		this.artifactsMap={};
		
		for (i=0; i<parsedResult.resources.length; i++) {
			var entry = this._createArtifact(parsedResult.resources[i]);
			entries.push(entry);
			this.artifactsMap[entry.uri] = entry;
		}
	
		this._treeify(entries);
		this.artifacts = entries;
		this.isLoaded = true;
		this.onLoad(args);
		return this;
	},
	
	getDeepestLevel: function () {
		// summary:
		//		Returns the level of the deepest items in the
		//		module structure. 
		return this._deepestLevel;
	},
	
	getArtifacts: function() {
		// summary:
		//		return the list of artifacts for display, this filters
		//		out anything that has a collapsed ancestor
		//		TODO: obviously a generic filter mechanism could be 
		//		   factored out of this. 
		
		// keeps track of the deepest item in the tree
		var retVal = dojo.filter(this.artifacts, function (artifact) {
			var parent = artifact.parent,
				hidden = false;
			while (parent && !hidden) {
				hidden = parent.collapsed;
				parent = parent.parent;
			}

			return !hidden;
		});
		
		if (retVal.length !== this.currentArtifactCount) {
			this.currentArtifactCount = retVal.length;
			this.onCountChanged();
		}
		
		return retVal;
	},
	
	getLast: function () {
		// summary:
		//		Get the very last entry in the module
		if (this.artifacts) {
			var length = this.artifacts.length;
			if (length > 0) {
				return this.artifacts[length - 1];
			}
		}
	},
			
	getArtifactByURI: function (uri) {
		return this.artifactsMap[uri];
	},
			
	getArtifactBySectionNumber: function (sectionNumber) {
		// summary:
		//		Looks for the artifact for the given sectionNumber.  If the module is in book order,
		//		attempts to find a parent section that does exist if the given section is not
		//		found.
		// returns:
		//		Returns properties indicating the uri and the section number of the item that was found.
		//		{ 
		//			uri: <value>,
		//			sectionNumber: <value>
		//		}
		
		var i; // loop counter
		
		if (this.isBookSort()) {
			// if the module is in bookorder, traverse the module hierarchy looking for the section
			// or something close to it
			var rootNodes = [];
			for (i=0; i<this.artifacts.length; i++) {
				var artifact = this.artifacts[i];
				if (!artifact.parentBinding) {
					rootNodes.push(artifact);
				}
			}
			
			// function to look for section within a set of nodes
			var foundEntry = null;
			var lookForSection = function(nodes, section) {
				var found= null;
				for (var i=0; i<nodes.length; i++) {
					if (nodes[i].sectionNumber === section) {
						found = nodes[i];
						break;
					}
				}
				return found;
			};
			
			var sections = sectionNumber;
			var findSection = null;
			var lastFoundEntry = null;
			var done = false;
			var index = sections.indexOf('.');
			var findNodes = rootNodes;
			
			while (!done && index !== -1) {
				var nextSection = sections.substring(0, index);
				sections = sections.substring(index+1);
				if (findSection != null) {
					findSection = findSection + '.' + nextSection;
				} else {
					findSection = nextSection;
				}
				foundEntry = lookForSection(findNodes, findSection);
				if (foundEntry) {
					lastFoundEntry = foundEntry;
					index = sections.indexOf('.');
					findNodes = foundEntry.children;
				} else {
					done = true;
				}
			}
			
			if (!done) {
				// look for last section
				if (findSection != null) {
					findSection = findSection + '.' + sections;
				} else {
					findSection = sections;
				}
				foundEntry = lookForSection(findNodes, findSection);
				if (foundEntry) {
					lastFoundEntry = foundEntry;
				} 
			}
			
			if (lastFoundEntry) {
				// TODO - in non-match case, could look at the children of the lastFoundEntry to try
				// to get nearer to what the user entered
				return {
					uri: lastFoundEntry.uri,
					sectionNumber: lastFoundEntry.sectionNumber
				};
			} else {
				return {
					uri: null,
					sectionNumber: null
				};
			}
		} else {
			for (i=0; i<this.artifacts.length; i++) {
				var artifact = this.artifacts[i];
				if (artifact.sectionNumber == sectionNumber) {
					return {
						uri: artifact.uri,
						sectionNumber: artifact.sectionNumber
					};
				}
			}
			return {
				uri: null,
				sectionNumber: null
			};
		}
	},
			
	getArtifactById: function (/*String*/artifactId) {
		// summary:
		//		Search for artifact by ID.
		//		artifact.id is also a string so make sure that you 
		//		pass the right type.
		for (var i=0; i<this.artifacts.length; i++) {
			var artifact = this.artifacts[i];

			if (artifact.id === artifactId) {
				return artifact.uri;
			}
		}
		return null;
	},
	
	isBookSort: function() {
		return (this.getPrimarySortBy().def === RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BookOrder);
	},
	
	countDescendants: function (node) {
		// summary:
		//		Count the number of descendant nodes, recursively counts the
		//		children.
		var total = (node.children ? node.children.length : 0),
			idx,
			iters = total;
				
		for (idx = 0; idx < iters; idx++) {		
			total += this.countDescendants(node.children[idx]);
		}
		return total;
	},
	
	getDescendants: function (node) {
		// summary:
		//		Recursively gather up all descendants of the
		//		given node/
		var descendants = [],
			recurseChildren = dojo.hitch(this, function (node) {
				var i;
				for (i=0; i<node.children.length; i++) {
					descendants.push(node.children[i]);
					recurseChildren(node.children[i]);
				}
			});
		recurseChildren(node);
		return descendants;
	},
	
	isAnyAnAncestor: function(artifactURI, resourceURIs){
		// summary:
		//		Used for checking to prevent user from attempting to make node 1.1
		//		the parent of node 1.
		// artifactURI:
		//		Target URI to determine if any are amongst its ancestors.
		// resourceURIs:
		//		Array of URIs to scan against.
		// returns:
		//		Returns true/false
		var pathToRoot = [];
		
		var parentURI = this.artifactsMap[artifactURI].parentBinding;
		var parentNode = this.artifactsMap[parentURI];
		
		while (parentURI && parentURI.length > 0 && parentURI !== this.compositeRootURI){
			pathToRoot.push(parentURI);
			parentURI = this.artifactsMap[parentURI].parentBinding;
			parentNode = this.artifactsMap[parentURI];
		}
		
		var foundAncestor = false;
		var index = 0;
		while (index < resourceURIs.length && !foundAncestor){
			var thisURI = resourceURIs[index];
			if (dojo.indexOf(pathToRoot, thisURI) !== -1){
				foundAncestor = true;
				break;
			}else{
				index++;
			}
		}
		return foundAncestor;
	},
	
	indexOf: function (entry) {
		return dojo.indexOf(this.artifacts, entry);
	},
	
	getArtifactCount: function () {
		return this.artifacts.length;
	},
	
	getTotalArtifactCount: function() {
		return this._totalArtifactsInModule;
	},
	
	getCurrentArtifactCount: function() {
		return this.currentArtifactCount;
	},
	
	getParentURIFromResourceURIAndWhere: function(uri, where){
		if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE || 
			where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER){
			var resource = this.getArtifactByURI(uri);
			if (resource.parentBinding && resource.parentBinding.length > 0){
				uri =  resource.parentBinding;
			}
			else{
				uri = this.compositeRootURI;
			}
		}
		else if (!where || where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST){
			uri = this.compositeRootURI;
		}
		//Below is covered as default
		
		return uri;
		
	},
	
	getAncestors: function(resourceUri) {
		// summary:
		//		returns an array of the Ids of all ancestors of the given resourceId
		var ancestors = [],
			artifactEntry = this.getArtifactByURI(resourceUri);
		while (artifactEntry.parent) {
			artifactEntry = artifactEntry.parent;
			ancestors.push(artifactEntry.uri);
		}
		return ancestors;
	},

	
	getNextSibling: function (uri) {
		// summary:
		//		Get the next sibling of the artifact specified by "uri".
		// description:
		//		Typically for use in some sort of command that does an "AFTER".
		//		For example, in this scenario:
		//		    1
		//		     1.1
		//		      1.1.1
		//		     1.2
		//		    2
		//		if you were to pass 'uri' as "1" this function would return
		//		object 2.
		
		var me = this.artifactsMap[uri];
		if (!me) {
			console.error("Was asked for an entry not in this module");
		}
		
		var idx = this.indexOf(me);
		idx += this.countDescendants(me);
		return this.artifacts[idx + 1];
		
	},
	
	findProperParentURI: function(targetURI, where){
		var parentURI;
		switch(where){
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER:
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE:
				parentURI = this.artifactsMap[targetURI].parentBinding;
			break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW:
			default:
				parentURI = targetURI;
			break;
		}
		if (!parentURI){
			parentURI = this.compositeRootURI;
		}
		
		return parentURI;
	},
	
	getPrecedingArtifact: function(targetURI, where){
		var parent = this.getArtifactByURI(targetURI),
			artifactList = this.artifacts,
			idx,
			before;
		
		switch(where){
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER:
				if (parent.children) {
					var numChildren = this.countDescendants(parent);
					idx = dojo.indexOf(artifactList, parent);
					idx += numChildren;
				} else {
					idx = dojo.indexOf(artifactList, parent);
				}
				before = artifactList[idx];
				break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE:
				idx = dojo.indexOf(artifactList, parent);
				idx--;
				before = artifactList[idx];
				break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW:
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST:
			default:
				if (parent && parent.children) {
					var numChildren = this.countDescendants(parent);
					idx = dojo.indexOf(artifactList, parent);
					idx += numChildren;
				} else if (!parent){
					idx = artifactList.length - 1;
				}
				else{	
					idx = dojo.indexOf(artifactList, parent);
				}
				before = artifactList[idx];
				break;
		}
		return before;
	},
	
	getParentForIndent: function(targetURI, rowIds){
		var resource = this.getArtifactByURI(targetURI),
			artifactList = this.artifacts,
			idx,
			before;
		
		idx = dojo.indexOf(artifactList, resource);
		idx--;
		before = artifactList[idx];
		
		while (before && (before.depth > resource.depth + 1)){
			idx--;
			before = artifactList[idx];
		}
		
		if (before && before.depth < resource.depth){
			before = null;
		}
		return before;
	},
	
	getParentForOutdent: function(targetURI){
		var resource = this.getArtifactByURI(targetURI);
		var parent = this.getArtifactByURI(resource.parentBinding);
		
		return parent;
	},
	
	stripToLowestCommonAncestors: function(/*Array*/resourceURIs){
		//summary:
		//Removes URIs that already exist in the module, changing the original array
		//Returns an array of URIs that exist in the module to the caller
		var index = 0;
		while (index < resourceURIs.length){
			var thisURI = resourceURIs[index];
			var propertyNode = this.artifactsMap[thisURI];
			//Validate that this is already in the module
			if (propertyNode){
				var parentURI = propertyNode.parentBinding;
				var removal = false;
				//Break things down based on lowest common ancestor
				while (parentURI && parentURI.length > 0 && parentURI != this.compositeRootURI){
					//If a parent is already to be moved, remove this bit from the set to move
					if (dojo.indexOf(resourceURIs, parentURI) != -1){
						resourceURIs.splice(index, 1);
						removal = true;
						break;
					}
					else{
						parentURI = this.artifactsMap[parentURI].parentBinding;
					}
				}
				if (!removal){
					index++;
				}
			}
			else{
				index++;
			}
		}
		return resourceURIs;
		
	},
	
	_getPropertyNode: function(targetURI){
		var structure = this.structure.getModuleStructureProperties();
		return structure.resultsetResourcesMap[targetURI];
	},
	
	_getChildrenNode: function(targetURI){
		var structurePiece = this._getPropertyNode(targetURI);
		if (structurePiece){
			var structure = this.structure.getModuleStructureProperties();
			var childName = structurePiece.children;
			return structure.resultsetChildrenMap[childName];
		}
		else{
			return null;
		}
	},

	//args{
	//	targetURI,		Targetted resource
	//	resources,???    Either resources
	//  resourceURIs,??? or resourceURIs can be handed over
	//	where,			Relation to target (below, before, after)
	//	isMove			Whether or not it's a move vs new addition
	//  isCopy			Whether or not it's a copy,
	//	forceStructureVersionLoad:
	//					Forces the latest version of the structure (the version of the module resource) to be
	//					loaded without any preceeding version check on it.
	//	skipOperationQueuing:
	//					Indicates that the place operation should proceed without any attempt to queue the operation
	//					behind any other pending operations.  This is necessary if the place operation is being
	//					called as part of a bigger operation that has already been queued in this way.  If both the outer
	//					and the inner operation are queued, they deadlock against each other.
	//					When this flag is set the latest version of the structure (the version of the module resource) is
	//					not checked or refreshed, since the enclosing operation will have done that as a
	//					a result of queuing its operation.
	//  waitForIndexing:
	//					If true, the operation will not conclude until the moved artifacts have been indexed on the
	//					server.  Otherwise the operation will return once the move transaction has been concluded on the
	//					server which may be before the indexing of the moved artifacts has completed
	// }
	placeChildren: function (args) {
		var deferred = new dojo.Deferred();
		var resourceURIs;
		
		//all we really care about is resourceURIs anyway
		if (args.resources){
			resourceURIs = [];
			dojo.forEach(args.resources, function(thisResource){
				resourceURIs.push(thisResource.getResourceURI());
			});
		}else{
			resourceURIs = args.resourceURIs;
		}
		
		resourceURIs = this.stripToLowestCommonAncestors(resourceURIs);
		
		var structureUpdateCallback = dojo.hitch(this, function(){
			this._numberAndFireOnChange({
				addItem: true,
				move: true,
				uri: resourceURIs,
				resources: args.resources
			});
		});
		if (args.callback) {
			deferred.addCallback(this, function(){
				this._updateStructure(args);
				var callbackDeferred = args.callback(args);
				callbackDeferred.addBoth(this, function(){
					structureUpdateCallback();
				});
			});
		}
		else {
			this._updateStructure(args);
			structureUpdateCallback();
		}
		
		if ((args.where !== com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST) && 
			(args.where !== com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW && args.isMove) ||
				args.isMove){
			//Need to perform a shift to place in the proper index on the parent
			var parentNode = args.targetURI ? this.artifactsMap[args.targetURI] : this.compositeRootURI;
			var getDeferred,
				parentURI;
			if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW){
				parentURI = parentNode.uri;
			}
			else{
				parentURI = parentNode.parentBinding;
			}
			//Special load for context root
			if (parentURI === this.compositeRootURI || !parentURI){
				var parentResource = new com.ibm.rdm.web.model.resources.modules.Structure({
						structureURI: this.compositeRootURI,
						projectURI: this.projectURI
				});
				getDeferred = parentResource.load(true);
			}
			else{
				getDeferred = com.ibm.rdm.web.model.resources.ResourceManager.get({
					resourceURI: parentURI,
					skipCache: true
				});
			}
			getDeferred.addCallback(this, function(targetResource){
				var children = targetResource.getRequirementChildren(),
					currentAfterTargetURI = args.targetURI;
				
				var insertIndex = -1;
				dojo.forEach(resourceURIs, function(thisURI){
					//Check to see if this resource is already a child. If so, remove it before it is placed
					var existingIndex = dojo.indexOf(children, thisURI);
					if (existingIndex != -1){
						children.splice(existingIndex, 1);
						}
					
					if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW){
						if (insertIndex == -1) {
							insertIndex = 0;
						}
						else {
							insertIndex++;
						}
					}
					else{
						// after or before
						insertIndex = dojo.indexOf(children, currentAfterTargetURI);
						if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER){
							insertIndex++;
							currentAfterTargetURI = thisURI;
						}
					}
					
					if (insertIndex < children.length){
						children.splice(insertIndex, 0, thisURI);
					}
					else{
						children.push(thisURI);
					}
				});
				if (dojo.isFunction(targetResource.setSerializer)) {
					// do a partial update to the heading if possible (i.e. the target resource is not the
					// root)
					targetResource.setSerializer(RequirementSerializer.PARTIAL_UPDATE_CHILD_LIST);
				}

				var isStructure = (targetResource instanceof com.ibm.rdm.web.model.resources.modules.Structure);
				
				//Module structure is a special case. Need to persist it in this instance if we haven't seen it before
				//and if we have, we can just carry on as normal and compare its etag
				var continueWithSave = dojo.hitch(this, function(args, etag){
					targetResource.setETag(etag);
					var saveDeferred = targetResource.save({refreshAfterSave:false, skipLinkCheck: true});
					var nextDeferred;
					if (args.waitForIndexing) {
						nextDeferred = saveDeferred.then(dojo.hitch(this, function(resource) {
							 return IndexingValidator.waitFor(resource);
						 }));
					} else {
						nextDeferred = saveDeferred;
					}
					nextDeferred.then(dojo.hitch(this, function () {
						//update in-memory etag for future edits
						if (isStructure){
							this.structureResource = targetResource;
						}
						else{
							this.getArtifactByURI(targetResource.resourceURI).eTag = targetResource.getETag();
						}
						
						//Move operations not only impact the target, but also the resourceURIs being moved
						//For now we must perform a separate call in order to update these post-operation
						//In the future this should be inlcuded in responses for calls which effect structure
						//~ebcraig
						if (args.isMove){
							this.updateETagOfResources(resourceURIs).then(dojo.hitch(this, function(updateResp){
								deferred.callback();
							}));
						}
						else{
							deferred.callback();
						}
					}), dojo.hitch(this, function(error){
						MessageHandler.showError(7741);
						deferred.errback(error);
					}));
					return saveDeferred;
				});
				
				var etag = null;

				if (isStructure){
					//Structure can be reliabily compared in-memory with time stamp on the module version loaded.
					//This situation will be ran into upon the first compositeRoot structure change of a loaded module.
					if (!this.structureResource && !args.skipOperationQueuing && dojo.isFunction(this.checkLatestTimestamp)){
						var latestDeferred = new dojo.Deferred();
						this.checkLatestTimestamp(latestDeferred, args.forceStructureVersionLoad, targetResource).then(
						dojo.hitch(this, function(shouldContinue){
							if (shouldContinue){
								continueWithSave(args, targetResource.getETag()).then(dojo.hitch(this, function(saveResp){
									latestDeferred.callback(saveResp);
								}), dojo.hitch(this, function(error){
									latestDeferred.errback(error);
								}));
							}
							else{
								latestDeferred.callback();
								MessageHandler.showError(7741);
								deferred.errback();
							}
						}));
					}
					else{
						etag = this.structureResource.etag;
					}
				}
				else{
					//Set etag to previously known version for this artifact
					etag = this.getArtifactByURI(targetResource.resourceURI).eTag;
				}
				
				if (etag){
					//TEMPORARY Measure until defect 70681 is resolved. Special isStructure case should be removed then -ebcraig
					if (!args.skipOperationQueuing && !isStructure && etag !== targetResource.getETag()){
						this.displayMergeConflict(targetResource, true).addCallback(this, function(dialogResponse) {
							if (dialogResponse){
								continueWithSave(args, targetResource.getETag());
							}
						});
					}
					else{
						//TEMPORARY Measure until defect 70681 is resolved. Special isStructure case should be removed then -ebcraig
						continueWithSave(args, etag);
					}
				}
				else if (!isStructure){
					MessageHandler.showError(7741);
					deferred.errback();
				}
				
			});
			getDeferred.addErrback(this, function(error){
				MessageHandler.showError(7741);
				deferred.errback(error);
			});
		} else {
			var updateParentEtag = dojo.hitch(this, function(effectedParentURI){
				this.updateETagOfResources([effectedParentURI]);
			});
			//Resource stubs need to be handled separately, particularly those from create similar
			if (args.resources && args.resources[0].stubId){
				// 1. Look for similarResource - what was selected for 'create similar' previously
				if (args.resources[0].similarResource){
					args.targetURI = args.resources[0].similarResource.resourceURI;
				}
				// 2. Check to see if the 'similar resource' still has a pending PUT or POST to server
				var artifactSaveDeferreds =
					args.targetURI ? EditManager.artifactBeingSaved(args.targetURI) : null;
				args.resources[0].getInitialSaveParams().saveCompleted = dojo.hitch(this, function(){
					// 2 (cont.) - If it does, hook onto the end of that for etag updates after save is done.
					if (artifactSaveDeferreds){
						artifactSaveDeferreds[artifactSaveDeferreds.length - 1].then(dojo.hitch(this, function(saveResponse){
							updateParentEtag(this.getParentURIFromResourceURIAndWhere(saveResponse.resourceURI, args.where));
						}));
					}
					// 2 (cont.) - If it does not, simply press forward with the known URI
					else{
						updateParentEtag(this.getParentURIFromResourceURIAndWhere(args.targetURI, args.where));
					}
				});
				deferred.callback();
			}
			else{
				// If we are not dealing with a stub, just update appropriate parent
				updateParentEtag(this.getParentURIFromResourceURIAndWhere(args.targetURI, args.where));
				deferred.callback();
			}
			
		}
		
		return deferred;
	},
	
	_toggleHeadingImpl: function (toggleArgs) {
		// summary:
		//		Toggles the headingness of 1 or more resources in 
		//		the module.  
		
		// an overall deferred which will fire when all the work 
		// this function does has been performed
		var overall = new dojo.Deferred();
					
		// Helper Function 
		var toggleHeadingOnDisplay = dojo.hitch(this, function(artifactURI, fireOnChange) {
			// summary:
			//		Toggles the heading flag on the entry in this ModuleStructure,
			var artifact = this.artifactsMap[artifactURI];
			artifact.isHeading = !artifact.isHeading;
			this.setIsHeading(artifactURI, artifact.isHeading, !fireOnChange);
		});
		
		var toggleMultiHeadingsOnDisplay = dojo.hitch(this, function(artifactURIs){
			// Helper to toggle several headings on display.
			dojo.forEach(artifactURIs, function(thisURI) {
				toggleHeadingOnDisplay(thisURI, false);
			}, this);
			// we passed false above to suppress the normal firing of the change event during
			// setting of the heading flag, we only want to do this once after we have changed
			// all the heading states.
			this._numberAndFireOnChange({
				move: true,
				uri: toggleArgs.resourceURIs
			});
		});
		
		// Helper Function
		var loadResources = function (resourceURIs) {
			// summary:
			//		Helper function to load all of the resources
			//		we want to toggle, returns a deferred list.
			var arrayOfDeferreds = [];
			dojo.forEach(resourceURIs, function (uri) {
				arrayOfDeferreds.push(com.ibm.rdm.web.model.resources.ResourceManager.get({
					resourceURI: uri,
					skipCache: true,
					reportErrors: true
				}));
			});
			
			return new dojo.DeferredList(arrayOfDeferreds);
		};
		
		var onceAllResourcesLoaded = dojo.hitch(this, function (results) {
			// summary:
			//		Invoked once all the resources are loaded, toggles the headingness
			//		and batch saves them all.
			// returns:
			//		A DeferredList response, containing error and response indication for
			//		each resource that was loaded.
			
			var saveDeferredArray = [], // a list of deferreds gotten from saving each resource. 
				saveDeferredList, // the dojo.DeferredList we'll make out of it
				resourcesToSave = [],	// An array of the resources being saved, ordered as saveDeferredArray.
				saveError,
				saveInitiatedDeferredArray = [], // a list of deferreds which complete when a save is initiated with RDMClient
				saveInitiatedDeferredList; // the deferredlist we make from them - which we need to 
				
			// iterate over the resources, toggling the heading and keeping a list of the resources to be saved.
			for (var i = 0 ; i < results.length ; i++) {
				var result = results[i],
					loaded = result[0],
					resource,
					loadError,
					errorType,
					isHeading;				
				if (loaded) {
					resource = result[1];
					// ask the module structure resource whether this resource
					// should be a heading - we updated the structure optimistically
					// before we loaded the resources which is why we can ask here.
					isHeading = this.getArtifactByURI(resource.resourceURI).isHeading;
					resource.setIsHeading(isHeading);
					//ensure that the etag used (which gets passed for If-Match header) 
					//is the one displayed to user - rather than which gets loaded on full resource fetch
					resource.setETag(this.getArtifactByURI(resource.resourceURI).eTag);
					resourcesToSave.push(resource);
				} else {
					// There was an error loading a resource
					loadError = result[1];
					errorType = loadError && loadError.err;
					var errorNumber = (errorType === com.ibm.rdm.web.model.resources.ResourceManager.NOT_FOUND) ? 7742 : 7667;
					MessageHandler.showError(errorNumber);
					// At least one of the loads failed - abort the whole operation.
					return loadError;
					
				}
			}
			
			// Now queue saves.  Do this after results from all loaded resources have been read so that we do not leave some saves
			// queued in the resource manager if there was an error loading one of the resources.
			
			// flip ResourceManager into batch save mode.
			com.ibm.rdm.web.model.resources.ResourceManager.startSaveBatching();
			dojo.forEach(resourcesToSave, function(resourceToSave){
				//Get a deferred so we know when this resource has had save initiated
				var saveInitiatedDeferred = new dojo.Deferred();
				saveInitiatedDeferredArray.push(saveInitiatedDeferred);
				// tell the resource that when it comes to serialize itself for
				// saving, it should only send the heading flag, see WI: 65720
				resourceToSave.setSerializer(RequirementSerializer.PARTIAL_UPDATE_HEADING);
				saveDeferredArray.push(resourceToSave.save({
					refreshAfterSave: false,
					skipLinkCheck: true,
					saveInitiatedDeferred: saveInitiatedDeferred
				}));
			}, this);
			
			
			saveDeferredList = new dojo.DeferredList(saveDeferredArray);
			saveDeferredList.then(dojo.hitch(this, function(resourcesToSave, results) {
				dojo.forEach(results, function (result, index) {
					var success = result[0],
						resource = result[1],
						resourceURI,
						isUnderEdit;
					if (!success) {
						// the save failed, so we need to rewind our optimistic update
						// of the UI.  Not got a valid saved resource, so use the pre-save
						// resource to derive the URI.
						resourceURI = resourcesToSave[index].resourceURI;
						toggleHeadingOnDisplay(resourceURI, true);
						saveError = result[1];
					} else {
						resourceURI = resource.resourceURI;
						isUnderEdit = EditManager.isArtifactUnderEdit(this.projectURI, resourceURI);
						if (isUnderEdit) {
							// Update the version of the artifact that is held in the edit manager, to match the
							// current server state.  We have already updated the heading state above, but do it
							// again (just to make sure!) and to set the revision.
							// We hold a lock for the artifact, so we can update the version information on the
							// instance in the edit manager, to avoid a version conflict when the edit manager
							// saves its instance.
							var isHeading = this.getArtifactByURI(resourceURI).isHeading;
							EditManager.setIsHeading(this.projectURI, resourceURI, isHeading, resource.revision);
							if (toggleArgs.onComplete) {
								toggleArgs.onComplete(resource, isUnderEdit);
							}
						}
						//update in-memory etag for future edits
						this.getArtifactByURI(resource.resourceURI).eTag = resource.getETag();
					}
					
				}, this);
				
				com.ibm.rdm.web.common.UpdateContext.cachedArtifactHasBeenSaved();
				if (saveError) {
					MessageHandler.showError(7740);
					overall.errback(saveError);
				} else {
					overall.callback();
				}
			}, resourcesToSave /*additional hitch parameter*/));
			// save them all...after save has been initiated
			saveInitiatedDeferredList = new dojo.DeferredList(saveInitiatedDeferredArray);
			saveInitiatedDeferredList.then(function() {
				com.ibm.rdm.web.model.resources.ResourceManager.completeSaveBatching();
			});
		});
		
		var onResourceLoadFailed = dojo.hitch(this, function(error){
			// Loading the resources has failed.  We optimistically toggled the rows on display, so
			// toggle them back again.
			toggleMultiHeadingsOnDisplay(toggleArgs.resourceURIs);
			overall.errback(error);
		});
		
		// onceAllResourcesLoaded will inspect the status of each individual GET in
		// the list of requests
		var loadDeferredList = loadResources(toggleArgs.resourceURIs);
		loadDeferredList.addCallback(onceAllResourcesLoaded);
		loadDeferredList.addErrback(onResourceLoadFailed);
		
		// Toggle the headings as displayed.
		toggleMultiHeadingsOnDisplay(toggleArgs.resourceURIs);
		
		return overall;
	},
	
	_toggleHeadingEditingStubImpl: function (toggleArgs) {
		// summary:
		//		This performs the toggling of headingness for artifacts that are still temporary stubs
		//		and which can't be udpated by the normal mechanism.  It changes the heading property
		//		of the artfiact and also updates the artifact information held in parallel by the
		//		EditManager.
		dojo.forEach(toggleArgs.resourceURIs, function(thisURI) {
			var isHeading = !this.getArtifactByURI(thisURI).isHeading;
			this.setIsHeading(thisURI, isHeading, true);
			EditManager.setIsHeading(this.projectURI, thisURI, isHeading);
		}, this);
		this._numberAndFireOnChange({
			move: true,
			uri: toggleArgs.resourceURIs
		});
		var overall = new dojo.Deferred();
		overall.callback();
		
		return overall;
	},
	
	toggleHeading: function (toggleArgs) {
		var latestDeferred = new dojo.Deferred();
		var isUnderEdit = EditManager.isArtifactUnderEdit(this.projectURI, toggleArgs.resourceURIs[0]);
		if (toggleArgs.isStub && isUnderEdit) {
			// If the artifact is a Stub artifact that has not yet been saved out we
			// cannot pass it through the standard updating mechanism, so we pass it
			// over to the special case handling function instead.
			var changeDeferred = this._toggleHeadingEditingStubImpl(toggleArgs);
			changeDeferred.addCallback(this, function(resp) {
				latestDeferred.callback(resp);
			});
		} else {
			var saveDeferred = this._toggleHeadingImpl(toggleArgs);
			saveDeferred.addCallback(this, function(saveResp) {
				latestDeferred.callback(saveResp);
			});
			saveDeferred.addErrback(this, function(error){
				latestDeferred.errback(error);
			});
		}
		return latestDeferred;
	},

	isHeading: function(artifactURI) {
		return this.getArtifactByURI(artifactURI).isHeading;
	},
		
	_updateStructure: function (args) {
		// summary:
		//		Update the list of artifacts, removing or adding things
		//		in the correct place depending on the operation.
		//		This function is responsible for maintaining the 
		//		clients view of the structure, so we need to make sure that
		//		all the artifacts have the correct parents and are inserted
		//		into the overall list in the correct place.
		
		// here's the list of things we need to find a new home for.
		var toParent = [],
			targetEntry = args.targetURI && this.artifactsMap[args.targetURI],
			idxOfTarget;
			
		if (args.isMove) {
			// if we are performing a move, remove the items
			// from their current places in the array of artifacts.
			if (args.resourceURIs) {
				dojo.forEach(args.resourceURIs, dojo.hitch(this, function (uri) {
					toParent = toParent.concat(this._removeItemFromParent(uri));
				}));
			} else {
				dojo.forEach(args.resources, dojo.hitch(this, function (resource) {
					toParent = toParent.concat(this._removeItemFromParent(resource.resourceURI));
				}));
			}
		} else if (args.isCopy) {
			// note by the time this is called, we have already created the
			// structure entries.
			dojo.forEach(args.resourceURIs, dojo.hitch(this, function (uri) {
				toParent = toParent.concat(this.artifactsMap[uri]);
			}));
		} else {
			dojo.forEach(args.resources, dojo.hitch(this, function(resource){
				// "createArtifact" is actually creating a simple
				// object with the stuff we need to keep from the resource.
				// this is the format of objects in this.artifacts
				toParent.push(this._createArtifact(resource));
			}));
		}
		
		// firstly, give all the things an entry in our map
		dojo.forEach(toParent, dojo.hitch(this, function (entry) {
			this.artifactsMap[entry.uri] = entry;
		}));

		idxOfTarget = targetEntry ? this.indexOf(targetEntry) : 0; 

		// now put the resources in the correct place in this.artifacts (the flat list of
		// everything in the module according to the view)
		if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
			|| args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST) {
			if (targetEntry) {
				idxOfTarget += this.countDescendants(targetEntry);
				this._insert(idxOfTarget + 1, toParent);
			} else {
				this.artifacts = this.artifacts.concat(toParent);
			}
		} else if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW) {
				this._insert(idxOfTarget + 1, toParent);
		} else {
			if (targetEntry) {
				this._insert(idxOfTarget, toParent);
			} else {
				this._insert(0, toParent);
			}
		} 
		
		
		// finally, set up the correct parentBinding, and set parent/children references
		if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW) {
			var toAddToFront = [];
			dojo.forEach(toParent, dojo.hitch(this, function (entry) {
				
				entry.parentBinding = targetEntry ? targetEntry.uri : this.compositeRootURI;
				if (targetEntry) {				
					toAddToFront.push(entry);
					entry.parent = targetEntry;
					targetEntry.collapsed = false;
					this._updateDepthRecursive(entry, targetEntry.depth);
				} else {
					this._updateDepthRecursive(entry, 0);
				}
			}));
			
			if (targetEntry) {
				targetEntry.children = toAddToFront.concat(targetEntry.children);
			}
		} else {
			var indexOfTarget = -1;
			dojo.forEach(toParent, dojo.hitch(this, function (entry) {
				entry.parentBinding = targetEntry ? targetEntry.parentBinding : this.compositeRootURI;
				var parent = this.artifactsMap[entry.parentBinding],
					parentsChildren = parent && parent.children;
				if (parent) {
					// find the index of the target or reference node in it's parent
					if (indexOfTarget == -1) {
						indexOfTarget = dojo.indexOf(parentsChildren, targetEntry);
					}

					if (args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER ||
						args.where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST) {
						indexOfTarget += 1;
					}
					parentsChildren.splice(indexOfTarget, 0, entry);
					this._updateDepthRecursive(entry, parent.depth);
					entry.parent = parent;
				} else {
					this._updateDepthRecursive(entry, 0);
				}	
			}));
		}
	},

	//Used to update resource in structure on save. 
	// replace a 'stub' resource (one which is not yet created on the server) with
	// a real resource (which has just been created on the server)
	_updateOnSave: function(args) {
		
		var updateEtag = dojo.hitch(this, function(uri, resource) {
			var oldResource = this.artifactsMap[uri],
				doUpdateArtifactsMap = oldResource && oldResource.uri === uri,
				stubIdOfUserUnmodifiableRow = resource.stubIdOfUserUnmodifiableRow;

			// The following 'if' is here to address Defect 83704.  We account for
			// the fact that the user has created a "stub row" in which team area
			// permissions checking has found that the none of the currently
			// displayed attributes are editable.  In this case the normal flow
			// of making use of the EditManager to do our "saving" is bypassed.
			// Instead, after creating the stub we immediately save the row
			// (in _saveUnmodifiableStubRow from ModuleArtifactCommands) and here
			// we account for this by appropriately updating certain class instance
			// fields.
			if (!doUpdateArtifactsMap && stubIdOfUserUnmodifiableRow) {
				oldResource = this.artifactsMap[stubIdOfUserUnmodifiableRow];
				doUpdateArtifactsMap = true;
			}

			if (doUpdateArtifactsMap){
				var newResource = resource;
				oldResource.uri = newResource.getResourceURI();
				oldResource.eTag = newResource.etag;
				oldResource.id = newResource.getID();
				oldResource.type = newResource.getType();
				oldResource.wrappedResourceMimeType = (dojo.isFunction(newResource.getWrappedResourceMimeType) ? 
					newResource.getWrappedResourceMimeType() : null);
				oldResource.wrappedResourceURI = (dojo.isFunction(newResource.getWrappedResourceURI) ? 
					newResource.getWrappedResourceURI() : null);

				// WI: 76131, if the headingness changed due to a change
				// of artifact type, renumber,
				var oldHeadingState = oldResource.isHeading;
				oldResource.isHeading = resource.isHeading();
				if (oldResource.isHeading !== oldHeadingState) {
					this._numberAndFireOnChange({
						uri: oldResource.uri
					});
				}
				//Remove reference to old ID, and set for new one.
				if (stubIdOfUserUnmodifiableRow) {
					delete this.artifactsMap[stubIdOfUserUnmodifiableRow];
				} else {
					delete this.artifactsMap[uri];
				}
				this.artifactsMap[oldResource.uri] = oldResource;
			}
		});
		
		
		if (args && args.notifierParams && args.notifierParams.resourceURI) {
			//Find and swap new properties from newly created/saved resource
			updateEtag(args.notifierParams.resourceURI, args.notifierParams.resources[0]);
		} else if (args && args.notifierParams && args.notifierParams.resources) {
			dojo.forEach(args.notifierParams.resources, function(resource) {
				updateEtag(resource.resourceURI, resource);
			});
		}
	},
	
	_updateDepthRecursive: function(entry, parentDepth) {
		var idx = 0;
		entry.depth = parentDepth + 1;
		
		for (idx; idx < entry.children.length; idx++) {
			this._updateDepthRecursive(entry.children[idx], entry.depth);
		}

		this._deepestLevel = Math.max(this._deepestLevel, entry.depth);
		
		return;
	},
	
	_insert: function (idx, artifacts) {
		// summary:
		//		Insert artifacts (an array of simple entry objects
		//		as per this._createArtifact()) into our list.
		//		This function is used by the update logic.
		//		After e.g. a move, we have an array of the artifacts
		//		that the move operation has been performed on, but 
		//		not their children.  
		//		The job of this function to insert the children into the flat
		//		list of items.
		var flatList = [],
			addChildren = function (to, entry) {
				// helper fn to recursively add children
				// into a flat list.
				var idx = 0;
				to = to.concat([entry]);
				for (idx; idx < entry.children.length; idx++) {
					to = addChildren(to, entry.children[idx]);
				}
				
				return to;
			};
		
		dojo.forEach(artifacts, function (artifact) {
			// recursively add all the children
			flatList = addChildren(flatList, artifact);
		});

		// some JS magic, is you supply an array directly to splice it
		// will insert he array into the array and not add each of the items
		// as we want.
		Array.prototype.splice.apply(this.artifacts, [idx, 0].concat(flatList));
	},
	
	_removeItemFromParent: function (itemUri, /*Boolean?*/removeFromMap) {
		// summary:
		//		Remove the item indicated by the given uri
		//		from it's parents children[] and from this.artifacts
		
		// get the item
		var entry = this.artifactsMap[itemUri],
			idxOfEntryInChildren,
			idxOfEntryInList;
			
		if (!entry) {
			console.error("Attempted to remove from module item that is not in it");
			return;
		}
		
		if (entry.parent) {
			// remove the entry from this list of children
			// in his parent.
			idxOfEntryInChildren = dojo.indexOf(entry.parent.children, entry);
			entry.parent.children.splice(idxOfEntryInChildren, 1);
			delete entry.parent;
		}

		// remove the item itself from the list of entries.
		idxOfEntryInList = this.indexOf(entry);
		// calculation at the end of this line also removes all the descendants of
		// the entry
		//
		// This works ONLY if the module is in ascending book order as it obviously
		// assumes that the the entries descendants all have a lower index in this.artifacts
		// than the next sibling of the item being removed. 
		var retVal = this.artifacts.splice(idxOfEntryInList, 1 + (this.countDescendants(entry))),
			that = this;
		
		if (removeFromMap) {
			dojo.forEach(retVal, function(artifact){
				delete that.artifactsMap[artifact.uri];
			});
		}
		// only return the top level artifact removed, i.e. not it's children
		// they don't have to be reparented
		return [retVal[0]];
	},
	

	removeItem: function (itemUri, fireOnChange) {
		
		if (typeof fireOnChange === "undefined") {
			fireOnChange = true;
		}
		
		this._removeItemFromParent(itemUri, true);
		delete this.artifactsMap[itemUri];
		if (fireOnChange) {
			var URIs = [];
			URIs.push(itemUri);
			
			this._numberAndFireOnChange({
				addItem: false,
				uri: URIs
			});
		}	
	},
	
	checkStructureCurrentByParentAndUpdateETag: function(resourceURI, where, originalOperationCompleted){
		// summary:
		//		Performs currentness check based on a given resource and 'where' argument
		//		for a structure argument. Important for new placements of resourcs.
		//		If passed a deferred, will perform update once operation is complete
		//		against the calculated approprate etag comparison resource
		// resourceURI:
		//		Target resource from user action
		// where:
		//		Structure information. Before, below, after, last.
		var overallDeferred = new dojo.Deferred();
		resourceURI = this.getParentURIFromResourceURIAndWhere(resourceURI, where);
		
		var structureResource = this.getArtifactByURI(resourceURI);
		
		if (resourceURI && structureResource){
			var eTag = structureResource.eTag;
			var latestDeferred = com.ibm.rdm.web.model.resources.ResourceManager.checkIsLatestVersionEtag(resourceURI, eTag);
			latestDeferred.then(dojo.hitch(this, function(result) {
				if (result.isLatest) {
					// it is the latest, no need to take any action.
					overallDeferred.callback(true);
				} else {
					// the version being edited is not the latest, check whether the
					// resource viewer widget knows about a custom merge editor for 
					// the resource type it is editing 
					// no custom merge editor, so default to simple dialog
					this.displayMergeConflict(result.resource).addCallback(this, function(dialogResponse) {
						overallDeferred.callback(dialogResponse);
						return dialogResponse;
					});
				}
				return result;
			}), 
			dojo.hitch(this, function(error) {
				overallDeferred.errback(error);
				return error;
			}));
			
			if (originalOperationCompleted){
				originalOperationCompleted.then(dojo.hitch(this, function(){
					this.updateETagOfResources([resourceURI]);
				}));
			}
		}
		else{
			overallDeferred.callback(true);
		}
		return overallDeferred;
	},
	
	updateETagOfResources: function(resourceURIs){
		// summary:
		//		Performs head on specific resource and updates model in structure accordingly
		// resourceURIs:
		//		array of resource URIs to update
		return com.ibm.rdm.web.model.resources.ResourceManager.getLatestETags(resourceURIs).then(dojo.hitch(this, function(newETags){
			var resourceURI = null;
			for (resourceURI in newETags){
				if (this.artifactsMap[resourceURI]){
					this.artifactsMap[resourceURI].eTag = newETags[resourceURI].eTag;
				}
				else if (resourceURI === this.compositeRootURI){
					if (!this.structureResource){
						this.structureResource = {};
					}
					this.structureResource.etag = newETags[resourceURI].eTag;
				}
			}
		}));
	},
	
	/* args {
	 * 	addItem,		(boolean) true if adding an item, false if removing
	 * 	move,			(optional boolean) true if moving an item
	 * 	uri,			list of URIs being added, removed, or moved
	 * }
	 */
	_numberAndFireOnChange: function (args) {
		this._totalArtifactsInModule = this.artifacts.length;
		this.sectionNumberer.number(this.artifacts);
		this.onChange(args);
	},
	
	setIsHeading: function(targetURI, isHeading, /*Boolean?*/dontFireOnChange){
		// summary:
		//		Notifies the stucture that a resource has had its heading state changed.
		//		When the required chages to the structure have been made, onChange is
		//		called to notify any listeners.
		// targetURI:
		//		Identifier for the resource changed
		// isHeading:
		//		Flag indicating the new heading state of the resource.
		// dontFireOnChange:
		//		Boolean as the name suggests if true don't trigger the change event
		//		
		var entry = this.artifactsMap[targetURI];
		if (entry) {
			entry.isHeading = isHeading;
			if (!dontFireOnChange) {
				this._numberAndFireOnChange({
					move: true,
					uri: [targetURI]
				});
			}
		}
	},
	
	onAddArtifacts: function() {
		
	},
	
	updatePrimaryTextForResources: function(resources) {
		dojo.forEach(resources, dojo.hitch(this, function (resource) {
			var uri = resource.resourceURI,
				entry = this.getArtifactByURI(uri);
			if (entry) {
				entry.summary = DomUtils.cleanHTMLString(resource.getPrimaryText(), {
					truncateTo: 30,
					separator: " "
				});
				entry.title = resource.getTitle();
			}
		}));
	},
	
	isCollapsedSection: function (uri) {
		// summary:
		//		Checks if the artifact specified by "uri" is 
		//		in collapsed state	
		var isCollapsed = false;
		var entry = this.getArtifactByURI(uri);
		if (entry) {
			isCollapsed = entry.collapsed ? true : false;
		}
		return isCollapsed;
	},
	
	collapseOrExpandSection: function (uri) {	
		// summary:
		//		Toggles the collapsed state of the artifact specified 
		//		by "uri"	
		var entry = this.getArtifactByURI(uri);
		if (entry) {
			entry.collapsed = entry.collapsed ? false : true;
		}
		// fire on change - this causes the page model to reindexed - or recreated
		// triggering a request to getArtifacts() - which will filter out things
		// with a collapsed ancestor.
		this.onChange();		
	},
	
	collapseOrExpandSectionList: function (uris) {	
		// summary:
		//		Toggles the collapsed state of a list of artifacts specified 
		//		by "uri"
		while (uris.length > 0) {
			var uri = uris.pop();
			var entry = this.getArtifactByURI(uri);
			if (entry) {
				entry.collapsed = entry.collapsed ? false : true;
			}
		}
		// fire on change - this causes the page model to reindexed - or recreated
		// triggering a request to getArtifacts() - which will filter out things
		// with a collapsed ancestor.
		this.onChange();		
	},
	
	collapseAllGreaterThan: function (level) {
		// summary:
		//		Sets the collapsed state on all nodes with a depth
		//		greater than level.
		var entry;
		level = parseInt(level, 10);
		for (var i=0; i<this.artifacts.length; i++) {
			entry = this.artifacts[i];
			// reset..
			entry.collapsed = false;
			if (entry.depth >= level) {
				entry.collapsed = true;
			}
		}
		
		this.onChange();	
	},
	
	arraySortOnBookorder: function(uriArray){
		// summary:
		//		Sorts the given array such that its entries are in book order.
		// uriArray:
		//		An array of URIs that are in this module.
		// returns:
		//		The modified array, for convenience.  The uriArray itself
		//		is modified in place.
		var self = this;
		uriArray.sort(function(a, b){
			return self.indexOf(self.getArtifactByURI(a)) - self.indexOf(self.getArtifactByURI(b));
		});
		return uriArray;
	},
	
	arraySortOnReverseBookorder: function(uriArray){
		// summary:
		//		Sorts the given array such that its entries are in reverse book order.
		// uriArray:
		//		An array of URIs that are in this module.
		// returns:
		//		The modified array, for convenience.  The uriArray itself
		//		is modified in place.
		var self = this;
		uriArray.sort(function(a, b){
			return self.indexOf(self.getArtifactByURI(b)) - self.indexOf(self.getArtifactByURI(a));
		});
		return uriArray;
	},

	destroy: function(){
		ChangeNotifier.removeListeners(this._saveListeners);
		this._saveListeners = null;
	},
	
	setCurrentModuleTreeLevel: function(level) {
		// summary:
		//		Set the current module levels to display.
		//		so that we can restore the levels to display after refresh.
		this._currentTreeLevel = level;
	},
	
	getCurrentModuleTreeLevel: function() {
		// summary:
		//		Retuen the current module levels to display.
		return this._currentTreeLevel;
	},
	
	getCollapsedAndExpandedArtifactUris: function(collapsedArtifactUris, expandedArtifactUris) {
		// summary:
		//		Get the collapsed and/or expanded state of artifacts.
		//		So that we can restore the states after refresh.
		var maxDepth = this.getDeepestLevel();
		if (this.artifacts) {
			for (var i=0; i < this.artifacts.length; i++) {
				var artifact = this.artifacts[i];
				if (artifact.parent) {
					if (this._currentTreeLevel == -1 || this._currentTreeLevel == 999 || this._currentTreeLevel == maxDepth) {
						// Get all the collapsed artifacts
						if (artifact.parent.collapsed) {
							if (dojo.indexOf(collapsedArtifactUris, artifact.parent.uri) === -1){
								collapsedArtifactUris.push(artifact.parent.uri);
							}
						}
					}
					else if (this._currentTreeLevel == 1) {
						// Get all the expanded artifacts
						if (!artifact.parent.collapsed) {
							if (dojo.indexOf(expandedArtifactUris, artifact.parent.uri) === -1){
								expandedArtifactUris.push(artifact.parent.uri);
							}
						}
					}
					else {
						// Get the collapsed and expanded artifacts
						if (artifact.parent.collapsed) {
							// Get all the collapsed artifacts
							if (dojo.indexOf(collapsedArtifactUris, artifact.parent.uri) === -1){
								collapsedArtifactUris.push(artifact.parent.uri);
							}
						}
						else {
							// Get all the expanded artifacts
							if (dojo.indexOf(expandedArtifactUris, artifact.parent.uri) === -1){
								expandedArtifactUris.push(artifact.parent.uri);
							}
						}
					}
				}
			}
		}
		if (expandedArtifactUris.length > 0) {
			expandedArtifactUris.reverse();
		}
	}
});

com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW = "below";
com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE = "before";
com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER = "after";
com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST = "last";
} ());
