/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
// NOTE: THIS IS A GENERATED FILE. DO NOT EDIT DIRECTLY!
dojo.provide("com.ibm.team.apt.web.ui.internal.common.viewer.plan.board.AbstractBoard");

dojo.require("com.ibm.jdojo.lang.Exception");
dojo.require("com.ibm.jdojo.lang.JavaScriptException");
dojo.require("com.ibm.jdojo.util.Assert");
dojo.require("com.ibm.team.apt.web.ui.internal.common.viewer.plan.gadgets.factories.ResizeableColumnGadgetFactory");
dojo.require("com.ibm.team.rtc.foundation.web.ui.gadgets.AbstractDojoObjectHTMLGadget");
dojo.require("com.ibm.team.rtc.foundation.web.ui.gadgets.factories.LabelGadgetFactory");
dojo.require("com.ibm.team.rtc.foundation.web.ui.gadgets.table.styled.ColumnResizeHelper");
dojo.require("com.ibm.team.rtc.foundation.web.ui.gadgets.table.styled.StyledTable");
dojo.require("com.ibm.team.rtc.foundation.web.ui.gadgets.table.styled.StyledTableColumn");

(function() {
var jdojo= com.ibm.jdojo;
var lang= jdojo.lang;
var Exception= lang.Exception;
var JavaScriptException= lang.JavaScriptException;
var Assert= jdojo.util.Assert;
var team= com.ibm.team;
var ResizeableColumnGadgetFactory= team.apt.web.ui.internal.common.viewer.plan.gadgets.factories.ResizeableColumnGadgetFactory;
var gadgets= team.rtc.foundation.web.ui.gadgets;
var AbstractDojoObjectHTMLGadget= gadgets.AbstractDojoObjectHTMLGadget;
var LabelGadgetFactory= gadgets.factories.LabelGadgetFactory;
var styled= gadgets.table.styled;
var ColumnResizeHelper= styled.ColumnResizeHelper;
var StyledTable= styled.StyledTable;
var StyledTableColumn= styled.StyledTableColumn;

/**
 * A class used by KanbanBoard and Taskboard
 */
var AbstractBoard= dojo.declare("com.ibm.team.apt.web.ui.internal.common.viewer.plan.board.AbstractBoard", AbstractDojoObjectHTMLGadget, {

	/**
	 * The Table which this board is controlling
	 */
	//fBoard: null,

	/**
	 * The model the board belongs to
	 */
	//fModel: null,

	/**
	 * The board to create
	 * 
	 * @param id
	 *            The id of the board
	 * @param model
	 *            The model the board belongs to
	 */
	constructor: function(id, model) {
		this.inherited(arguments, []);
		this.fModel= model;
		this.fBoard= new StyledTable(id);
		this.fBoard.addClass(AbstractBoard.CLASS_NAME);
		this.fBoard.setParent(this);
	},

	/**
	 * Gets a state group from the board
	 * 
	 * @param stateGroup
	 *            The state group to retrieve
	 * @return A TableColumn associated with the state
	 */
	getStateGroup: function(stateGroup) {
		return this.fBoard.getColumn(AbstractBoard.getStateGroupId(stateGroup));
	},

	/**
	 * Gets a state from the board
	 * 
	 * @param state
	 *            The state to retrieve, null to get the state group hiding
	 *            states column
	 * @param stateGroup
	 *            The state group the state belongs to
	 * @return A TableColumn associated with the state
	 */
	getState: function(state, stateGroup) {
		return this.fBoard.getColumn(AbstractBoard.getStateId(state, stateGroup));
	},

	/**
	 * Tracks the given board as a group nested below this board
	 * 
	 * @param dependentBoard
	 * 			The board to track
	 */
	addDependentBoard: function(dependentBoard) {
		this.setIsCollapsible(true);
		this.fBoard.addDependentTable(dependentBoard.getBoard());
	},

	/**
	 * Adds a state group to the board
	 * 
	 * @param stateGroup
	 *            The state group to add
	 * @return A TableColumn associated with the state group
	 */
	addStateGroup: function(stateGroupProperties) {
		var column= this.fBoard.addColumn(AbstractBoard.getStateGroupId(stateGroupProperties.stateGroup), null, new ResizeableColumnGadgetFactory().setInternalGadgetFactory(this.getStateGroupHeaderFactory(stateGroupProperties)), null);
		column.addClass(AbstractBoard.STATE_GROUP_CLASS);
		column.setIsResizable(true);
		column.setIsCollapsible(true);
		return column;
	},

	/**
	 * Get a gadget factory for the column header of a state group.
	 * 
	 * @param stateGroup
	 *            The state group associated with the column header
	 * @return a gadget factory appropriate for the board type
	 */
	getStateGroupHeaderFactory: function(stateGroup) {
		Assert.isTrue(false, "Abstract method 'getStateGroupHeaderFactory' invoked");
	},

	/**
	 * Adds a state to the board
	 * 
	 * @param state
	 *            The state to add. May not be null.
	 * @param stateGroup
	 *            The state group the state belongs to. May not be null. This
	 *            parameter does not need to be a real IWorkflowStateGroup but
	 *            must implement the methods getId() and getLabel(). Creates a
	 *            column for a state/state group. Can also be used to create the
	 *            story column of a task board.
	 * @return A TableColumn associated with the state
	 */
	addState: function(state, stateGroup) {
		var stateGroupColumn= this.fBoard.getColumn(AbstractBoard.getStateGroupId(stateGroup));
		var sibilingStateColumns= stateGroupColumn.getSubColumns();
		if (sibilingStateColumns.length > 0) {
			sibilingStateColumns[sibilingStateColumns.length - 1].removeClass(AbstractBoard.LAST_STATE_IN_STATE_GROUP_CSS_CLASS);
			sibilingStateColumns[sibilingStateColumns.length - 1].removeCellClass(AbstractBoard.LAST_STATE_IN_STATE_GROUP_CSS_CLASS);
		}
		var column= null;
		if (state == null) {
			column= this.fBoard.addColumn(AbstractBoard.getStateId(null, stateGroup), stateGroupColumn, new ResizeableColumnGadgetFactory().setInternalGadgetFactory(new LabelGadgetFactory("")), null);
		} else {
			column= this.fBoard.addColumn(AbstractBoard.getStateId(state, stateGroup), stateGroupColumn, new ResizeableColumnGadgetFactory().setInternalGadgetFactory(new LabelGadgetFactory(state.getLabel(), state.getIconUri())), null);
		}
		column.addClass(AbstractBoard.STATE_CLASS);
		column.addClass(AbstractBoard.LAST_STATE_IN_STATE_GROUP_CSS_CLASS);
		column.addCellClass(AbstractBoard.LAST_STATE_IN_STATE_GROUP_CSS_CLASS);
		column.setIsResizable(true);
		column.setIsCollapsible(true);
		return column;
	},

	/**
	 * Gets a specific cell gadget
	 * 
	 * @param column
	 *            The column to retrieve the gadget from
	 * @param row
	 *            The row to retrieve the gadget from
	 * @return The gadget assoicated with that row and column
	 */
	getCellGadget: function(column, row) {
		return column.getCellByRow(row).getGadget();
	},

	/**
	 * Adds a task note to a column and row
	 * 
	 * @param column
	 *            The column to add the task note to
	 * @param row
	 *            The row to add the task note to
	 * @param note
	 *            The task note to add
	 * @param position
	 *            Where in the cell to add the task note
	 */
	addTaskNote: function(column, row, note, position) {
		this.getCellGadget(column, row).addTask(note, position);
	},

	/**
	 * Gets the twistie for the board
	 * 
	 * @return The twistie on the board
	 */
	getTwistie: function() {
		var headerGadget= this.fBoard.getGroupHeader();
		if (headerGadget != null) {
			return headerGadget.getTwistie();
		}
		return null;
	},

	/**
	 * Gets the Board as a Table
	 * 
	 * @return The Table
	 */
	getBoard: function() {
		return this.fBoard;
	},

	getHTMLElement: function() {
		return this.fBoard.getHTMLElement();
	},

	isCollapsed: function() {
		return this.fBoard.hasClass(AbstractBoard.NON_HEADER_COLLAPSE_CLASS);
	},

	isCollapsible: function() {
		return this.fBoard.hasClass(AbstractBoard.COLLAPSIBLE_CLASS);
	},

	setIsCollapsed: function(isCollapsed) {
		this.fBoard.toggleClass(AbstractBoard.NON_HEADER_COLLAPSE_CLASS, isCollapsed);
		var twistie= this.getTwistie();
		if (twistie != null) {
			twistie.refreshGadget();
		}
		var $subject= this.fBoard.getDependentTables();
		var $length= $subject.length;
		for (var $count= 0; $count < $length; $count++){
			var table= $subject[$count];
			table.setIsHidden(isCollapsed);
		}
	},

	setIsCollapsible: function(isCollapsible) {
		this.fBoard.toggleClass(AbstractBoard.COLLAPSIBLE_CLASS, isCollapsible);
	},

	/**
	 * Associates a columnDescriptor with a state group
	 */
	setStateGroupProperties: function(stateGroupProperties) {
		Assert.isTrue(false, "Abstract method 'setStateGroupProperties' invoked");
	},

	/**
	 * Normalizes all of the column sizes on board load
	 */
	adjustColumnWidths: function() {
		var columns= this.getBoard().getColumns(null);
		if (columns == null) {
			return ;
		}
		var columnWidthArea= dojo.contentBox(this.getHTMLElement()).w;
		var usableColumns= [];
		var minimumColumnSize= 0;
		for (var i= 0; i < columns.length; i++){
			if (columns[i].getParentColumn() instanceof StyledTableColumn && columns[i].getParentColumn().isCollapsed()) {
				return ;
			} else if (columns[i] instanceof StyledTableColumn && columns[i].isCollapsed()) {
				return ;
			} else {
				if (minimumColumnSize === 0) {
					minimumColumnSize= ColumnResizeHelper.getColumnMinimumWidth(columns[i]);
				}
				columnWidthArea-= minimumColumnSize;
				dojo.style(columns[i].getHTMLElement(), 'width', minimumColumnSize + "px");
				if (ColumnResizeHelper.isResizable(columns[i])) {
					usableColumns.push(columns[i]);
				}
			}
		}
		if (columnWidthArea > 0) {
			var $subject2= usableColumns;
			var $length2= $subject2.length;
			for (var $count2= 0; $count2 < $length2; $count2++){
				var column= $subject2[$count2];
				dojo.style(column.getHTMLElement(), 'width', ((((columnWidthArea / usableColumns.length)|0)) + minimumColumnSize) + "px");
			}
		}
		var $subject3= this.getBoard().getColumns(0);
		var $length3= $subject3.length;
		for (var $count3= 0; $count3 < $length3; $count3++){
			var column= $subject3[$count3];
			var totalWidth= 0;
			var $subject4= column.getSubColumns();
			var $length4= $subject4.length;
			for (var $count4= 0; $count4 < $length4; $count4++){
				var subColumn= $subject4[$count4];
				var width= dojo.style(subColumn.getHTMLElement(), 'width');
				try {
					if (dojo.isString(width)) {
						width= width.substring(0, width.length - 2);
					}
					totalWidth+= parseInt(width);
				} catch (e) {
					if ((e= e instanceof Exception ? e : new JavaScriptException(e)) instanceof Exception) {
					} else {
						throw e;
					}
				}
			}
			dojo.style(column.getHTMLElement(), 'width', totalWidth + "px");
		}
	},

	destroy: function() {
		if (this.fBoard != null) {
			this.fBoard.destroy();
		}
	},

	$interfaces: {
		'com.ibm.team.rtc.foundation.web.IDestroyable': true,
		'com.ibm.team.rtc.foundation.web.ui.gadgets.ICollapsible': true
	}
});

AbstractBoard.LAST_STATE_IN_STATE_GROUP_CSS_CLASS= "lastStateInStateGroup";
AbstractBoard.CLASS_NAME= "com-ibm-team-apt-web-ui-internal-common-viewer-plan-board-AbstractBoard";
AbstractBoard.STATE_CLASS= "state";
AbstractBoard.STATE_GROUP_CLASS= "state-group";
AbstractBoard.NON_HEADER_COLLAPSE_CLASS= "nonGroupCollapse";
AbstractBoard.COLLAPSIBLE_CLASS= "collapsible";
AbstractBoard.getStateGroupId= function(stateGroup) {
	return "StateGroup <" + stateGroup.getId() + ">";
};
AbstractBoard.getStateId= function(state, stateGroup) {
	if (state == null) {
		return "StateGroup <" + stateGroup.getId() + "> No States";
	} else {
		return "StateGroup <" + stateGroup.getId() + "> State <" + state.getLabel() + ">";
	}
};

})();
